/*	$ DOOROOS: drv_gpio.c $	*/

/*
 * Copyright (C) 2016	iBOSN systems
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are not permitted completely.
 *
 */

/*
 * Module Name: drv_gpio.c
 *
 * Purpose: Start of DOOROOS Operating System.
 *          gpio device driver
 *
 */
#include <types.h>
#include <thread.h>
#include <message.h>
#include <interrupt.h>
#include <device.h>
#include <error.h>

#include <time.h>

#include <panic.h>

#include <mach/hal_time.h>

#include <dev/gpiodrv.h>

#include <mach/SAMA5D3X.h>
#include <mach/SAMA5D3Xbd.h>
#include <mach/gpio.h>



//==========================================================================

#define TIMER_ENABLE 1
#if TIMER_ENABLE

#define GPIO_TIMER_DELAY          1000
#define GPIO_TIMER_BITMAP         0x100
STATIC HANDLE       hGpioSoftTimer;

#endif

//==========================================================================

STATIC INT32
th_GpioDevice( UINT32 Arg )
{
    JB_RECVMESSAGE      Message;
    HANDLE              hPort;
    HANDLE              Handle = NULL;
    PDEVICECONFIG       pDeviceConfig;
    UINT32       Channel;
    
    pDeviceConfig = (PDEVICECONFIG)Arg;
    
    hPort = PortGet( );    
    PortSetType( hPort, PORTTYPE_MESSAGE/*|PORTTYPE_INTERRUPT*/|PORTTYPE_SIGNAL );

    Channel = ChannelGet( hPort );

    pDeviceConfig->Name[3] = (UINT32)pDeviceConfig->Minor + '1';
    pDeviceConfig->Channel = Channel;
    DeviceRegister( (PDEVICECONFIG)pDeviceConfig );

#if TIMER_ENABLE
    hGpioSoftTimer = DrvTimerCreate( Channel, GPIO_TIMER_DELAY, GPIO_TIMER_BITMAP );
    // start the timer
    if( hGpioSoftTimer != NULL )
        DrvTimerStart(hGpioSoftTimer); 
#endif

	while(1)
	{
		MessageReceive( hPort, &Message);

		switch(Message.Type)
		{
		case MESSAGE_TYPE_INTERRUPT:
            continue;
		case MESSAGE_TYPE_SIGNAL:
		{

#if TIMER_ENABLE		
            if( Message.Param & GPIO_TIMER_BITMAP )
            {
				STATIC UINT32 nGpio = 0;
				
				switch(nGpio)
				{
				case 0:
					break;
				case 1:
					break;
				}
            }

#endif
            continue;
		}
		
		case MESSAGE_TYPE_DEVICE_READ:
		case MESSAGE_TYPE_DEVICE_WRITE:
	    {
            Message.Type = 0;
            break;
	    }

		case MESSAGE_TYPE_DEVICE_CONTROL:
	    {
            PIOCONTROL_PARAM    pIoControlParam;

            if( Handle == NULL || Message.Handle != Handle )
            {
                ThreadSetLastError( Message.hSender, ERR_INVALID_HANDLE );
                Message.Type = FALSE;
                break;
            }

            pIoControlParam = (PIOCONTROL_PARAM)Message.Param;

		    switch( pIoControlParam->Cmd )
		    {
            case IOCTRL_GPIO_INIT:
            {
                GPIO_SET* pSet = (GPIO_SET*)pIoControlParam->pInBuf;

                struct pio_desc set_pio;
                set_pio.pin_name = pSet->pName;
                set_pio.pin_num = pSet->Num;
                set_pio.dft_value = pSet->Value;
                set_pio.attribute = (pSet->PullUp ? PIO_PULLUP : 0) |
                                    (pSet->PullDown ? PIO_PULLDN : 0) |
                                    (pSet->Deglitch ? PIO_DEGLITCH : 0) |
                                    (pSet->OpenDrain ? PIO_OPENDRAIN : 0) |
                                    PIO_DEFAULT;
                set_pio.type = pSet->bOutput? PIO_OUTPUT_1 : PIO_INPUT;
                pio_setup( &set_pio, 1);
                Message.Type = TRUE;
				break;
            }

            case IOCTRL_GPIO_SET:
            {
                UINT32 iPin = (UINT32)pIoControlParam->pInBuf;
                pio_set_value( iPin, 1 );
                Message.Type = TRUE;
				break;
            }

            case IOCTRL_GPIO_CLEAR:
            {
                UINT32 iPin = (UINT32)pIoControlParam->pInBuf;
                pio_set_value( iPin, 0 );
                Message.Type = TRUE;
				break;
				break;
            }
		    case IOCTRL_GPIO_GET:
            {
                UINT32 iPin = (UINT32)pIoControlParam->pInBuf;
                Message.Type = pio_get_value( iPin );
				break;
            }

		    default:
                Message.Type = FALSE;
				break;
			}
			break;
	    }

		case MESSAGE_TYPE_DEVICE_OPEN:
        {
            //Handle = ChannelOpenHandleGetFrom( Channel );
            Handle = Channel;
            Message.Type = Handle;

            break;
        }

		case MESSAGE_TYPE_DEVICE_CLOSE:
        {
		    Handle = NULL;
		    Message.Type = TRUE;
			break;
	    }

        default:
            break;
		}

        MessageReply(&Message);

	}

	return 0;	
}



//==========================================================================

#define GPIODEV_STACK_SIZE      (512)

STATIC DEVICECONFIG             GpioDeviceConfig ={"GPO", 0, -1, NULL, NULL };

PUBLIC THREADCONFIG             DeviceThread =
                                            {
                                                (UINT32)&GpioDeviceConfig,
                                                th_GpioDevice,
                                                (PBYTE)NULL,
                                                GPIODEV_STACK_SIZE,
                                                THREAD_PRIORITY_DRIVER_HIGH+11
                                            };
//==========================================================================

#include <moduleloader.h>

PUBLIC HANDLE 
DllMain( HANDLE hDll, UINT32 Reason, PVOID Reserved )
{
    HANDLE hThread = NULL;
    switch( Reason )
    {
    case DLL_PROCESS_ATTACH:
        if (Reserved)
        {
            PDRIVER_ARG pDriverArg = Reserved;
            if( pDriverArg->Minor == 0 )
                                        DeviceThread.Arg       = (UINT32)&GpioDeviceConfig;
            else
                break;

            DeviceThread.pFunc       = th_GpioDevice;

            if (pDriverArg->pStack)     DeviceThread.pStack    = (PBYTE)pDriverArg->pStack;
            if (pDriverArg->StackSize)  DeviceThread.StackSize = pDriverArg->StackSize;
            if (pDriverArg->Priority)   DeviceThread.Priority  = pDriverArg->Priority;
            hThread = ThreadCreate (&DeviceThread);
            //kprintf("Serial Thread %x\n", hThread);
        }
    	break;
    
    case DLL_PROCESS_DETACH:
    	break;
    }

    return hThread;
}




